@@ -2,10 +2,12 @@
 
 from __future__ import annotations
 
+from dataclasses import dataclass
 from datetime import date
 from datetime import timedelta
 from functools import cached_property
 from functools import lru_cache
+from itertools import islice
 from itertools import takewhile
 from os.path import normpath
 from pathlib import Path
@@ -30,12 +32,11 @@ from fava.core.attributes import AttributesModule
 from fava.core.budgets import BudgetModule
 from fava.core.charts import ChartModule
 from fava.core.commodities import CommoditiesModule
-from fava.core.conversion import cost_or_value
+from fava.core.conversion import conversion_from_str
 from fava.core.extensions import ExtensionModule
 from fava.core.fava_options import parse_options
 from fava.core.file import _incomplete_sortkey
 from fava.core.file import FileModule
-from fava.core.file import get_entry_slice
 from fava.core.filters import AccountFilter
 from fava.core.filters import AdvancedFilter
 from fava.core.filters import TimeFilter
@@ -57,6 +58,7 @@ if TYPE_CHECKING:  # pragma: no cover
     from collections.abc import Mapping
     from collections.abc import Sequence
     from decimal import Decimal
+    from typing import Literal
 
     from fava.beans.abc import Directive
     from fava.beans.types import BeancountOptions
@@ -92,6 +94,14 @@ class StatementMetadataInvalidError(FavaAPIError):
         )
 
 
+@dataclass(frozen=True)
+class JournalPage:
+    """A page of journal entries."""
+
+    entries: Sequence[tuple[int, Directive]]
+    total_pages: int
+
+
 class FilteredLedger:
     """Filtered Beancount ledger."""
 
@@ -99,6 +109,7 @@ class FilteredLedger:
         "__dict__",  # for the cached_property decorator
         "_date_first",
         "_date_last",
+        "_pages",
         "date_range",
         "entries",
         "ledger",
@@ -116,6 +127,14 @@ class FilteredLedger:
     ) -> None:
         self.ledger = ledger
         self.date_range: DateRange | None = None
+        self._pages: (
+            tuple[
+                int,
+                Literal["asc", "desc"],
+                list[Sequence[tuple[int, Directive]]],
+            ]
+            | None
+        ) = None
 
         entries = ledger.all_entries
         if account:
@@ -159,6 +178,11 @@ class FilteredLedger:
         entries.sort(key=_incomplete_sortkey)
         return entries
 
+    @cached_property
+    def entries_without_prices(self) -> Sequence[Directive]:
+        """The filtered entries, without prices for journals."""
+        return [e for e in self.entries if not isinstance(e, Price)]
+
     @cached_property
     def root_tree(self) -> Tree:
         """A root tree."""
@@ -211,6 +235,45 @@ class FilteredLedger:
             return False
         return close_date < date_range.end if date_range else True
 
+    def paginate_journal(
+        self,
+        page: int,
+        per_page: int = 1000,
+        order: Literal["asc", "desc"] = "desc",
+    ) -> JournalPage | None:
+        """Get entries for a journal page with pagination info.
+
+        Args:
+            page: Page number (1-indexed).
+            order: Datewise order to sort in
+            per_page: Number of entries per page.
+
+        Returns:
+            A JournalPage, containing a list of entries as (global_index,
+            directive) tuples in reverse chronological order and the total
+            number of pages.
+        """
+        if (
+            self._pages is None
+            or self._pages[0] != per_page
+            or self._pages[1] != order
+        ):
+            pages: list[Sequence[tuple[int, Directive]]] = []
+            enumerated = list(enumerate(self.entries_without_prices))
+            entries = (
+                iter(enumerated) if order == "asc" else reversed(enumerated)
+            )
+            while batch := tuple(islice(entries, per_page)):
+                pages.append(batch)
+            if not pages:
+                pages.append([])
+            self._pages = (per_page, order, pages)
+        _per_pages, _order, pages = self._pages
+        total = len(pages)
+        if page > total:
+            return None
+        return JournalPage(pages[page - 1], total)
+
 
 class FavaLedger:
     """Create an interface for a Beancount ledger.
@@ -235,6 +298,7 @@ class FavaLedger:
         "fava_options_errors",
         "file",
         "format_decimal",
+        "get_entry",
         "get_filtered",
         "ingest",
         "load_errors",
@@ -271,6 +335,7 @@ class FavaLedger:
         self.beancount_file_path = path
         self._is_encrypted = is_encrypted_file(path)
         self.get_filtered = lru_cache(maxsize=16)(self._get_filtered)
+        self.get_entry = lru_cache(maxsize=16)(self._get_entry)
 
         #: An :class:`AttributesModule` instance.
         self.attributes = AttributesModule(self)
@@ -316,6 +381,7 @@ class FavaLedger:
             is_encrypted=self._is_encrypted,
         )
         self.get_filtered.cache_clear()
+        self.get_entry.cache_clear()
 
         self.all_entries_by_type = group_entries_by_type(self.all_entries)
         self.prices = FavaPriceMap(self.all_entries_by_type.Price)
@@ -474,7 +540,7 @@ class FavaLedger:
         *,
         with_children: bool,
     ) -> Iterable[
-        tuple[Directive, SimpleCounterInventory, SimpleCounterInventory]
+        tuple[int, Directive, SimpleCounterInventory, SimpleCounterInventory]
     ]:
         """Journal for an account.
 
@@ -486,15 +552,16 @@ class FavaLedger:
                            the account.
 
         Yields:
-            Tuples of ``(entry, change, balance)``.
+            Tuples of ``(index, entry, change, balance)``.
         """
+        conv = conversion_from_str(conversion)
         relevant_account = account_tester(
             account_name, with_children=with_children
         )
 
         prices = self.prices
         balance = CounterInventory()
-        for entry in filtered.entries:
+        for index, entry in enumerate(filtered.entries_without_prices):
             change = CounterInventory()
             entry_is_relevant = False
             postings = getattr(entry, "postings", None)
@@ -509,12 +576,13 @@ class FavaLedger:
 
             if entry_is_relevant:
                 yield (
+                    index,
                     entry,
-                    cost_or_value(change, conversion, prices, entry.date),
-                    cost_or_value(balance, conversion, prices, entry.date),
+                    conv.apply(change, prices, entry.date),
+                    conv.apply(balance, prices, entry.date),
                 )
 
-    def get_entry(self, entry_hash: str) -> Directive:
+    def _get_entry(self, entry_hash: str) -> Directive:
         """Find an entry.
 
         Arguments:
@@ -542,8 +610,6 @@ class FavaLedger:
         Directive,
         Mapping[str, Sequence[str]] | None,
         Mapping[str, Sequence[str]] | None,
-        str,
-        str,
     ]:
         """Context for an entry.
 
@@ -557,10 +623,9 @@ class FavaLedger:
             the balances before and after the entry of the affected accounts.
         """
         entry = self.get_entry(entry_hash)
-        source_slice, sha256sum = get_entry_slice(entry)
 
         if not isinstance(entry, (Balance, Transaction)):
-            return entry, None, None, source_slice, sha256sum
+            return entry, None, None
 
         entry_accounts = get_entry_accounts(entry)
         balances = {account: Inventory() for account in entry_accounts}
@@ -577,12 +642,12 @@ class FavaLedger:
         before = {acc: visualise(inv) for acc, inv in balances.items()}
 
         if isinstance(entry, Balance):
-            return entry, before, None, source_slice, sha256sum
+            return entry, before, None
 
         for posting in entry.postings:
             balances[posting.account].add_position(posting)  # type: ignore[arg-type]
         after = {acc: visualise(inv) for acc, inv in balances.items()}
-        return entry, before, after, source_slice, sha256sum
+        return entry, before, after
 
     def commodity_pairs(self) -> Sequence[tuple[str, str]]:
         """List pairs of commodities.
